#ifdef MANUAL_ALLOC

extern char *spxGlobalHeapPtr, *spxGlobalHeapEnd;
extern char *spxGlobalScratchPtr, *spxGlobalScratchEnd;

/* Make sure that all structures are aligned to largest type */
#define BLOCK_MASK      (sizeof(long double)-1)

#define OVERRIDE_SPEEX_ALLOC
void *speex_alloc (int size)
{
    void *ptr;
    
    ptr = (void *) (((int)spxGlobalHeapPtr + BLOCK_MASK) & ~BLOCK_MASK);  //Start on 8 boundary

    spxGlobalHeapPtr = (char *)((int)ptr + size);	// Update pointer to next free location

    if (spxGlobalHeapPtr > spxGlobalHeapEnd ) 
    {
#ifdef VERBOSE_ALLOC
	    fprintf (stderr, "insufficient space for persistent alloc request %d bytes\n", size);
#endif
       return 0;
    }
   
#ifdef VERBOSE_ALLOC
    fprintf (stderr, "Persist Allocated %d chars at %x, %d remaining\n", size, ptr, ((int)spxGlobalHeapEnd - (int)spxGlobalHeapPtr));
#endif
    memset(ptr, 0, size);
    return ptr;
}

#define OVERRIDE_SPEEX_ALLOC_SCRATCH
void *speex_alloc_scratch (int size)
{
    void *ptr;

    ptr = (void *) (((int)spxGlobalScratchPtr + BLOCK_MASK) & ~BLOCK_MASK);  //Start on 8 boundary

    spxGlobalScratchPtr = (char *)((int)ptr + size);	// Update pointer to next free location

    if (spxGlobalScratchPtr > spxGlobalScratchEnd ) 
    {
#ifdef VERBOSE_ALLOC
	    fprintf (stderr, "insufficient space for scratch alloc request %d bytes\n", size);
#endif
       return 0;
    }
   
#ifdef VERBOSE_ALLOC
    fprintf (stderr, "Scratch Allocated %d chars at %x, %d remaining\n", size, ptr, ((int)spxGlobalScratchEnd - (int)spxGlobalScratchPtr));
#endif
    memset(ptr, 0, size);
    return ptr;
}

#define OVERRIDE_SPEEX_REALLOC
void *speex_realloc (void *ptr, int size)
{
#ifdef VERBOSE_ALLOC
   speex_warning("realloc attempted, not allowed");
#endif
   return 0;
}

#define OVERRIDE_SPEEX_FREE
void speex_free (void *ptr)
{
#ifdef VERBOSE_ALLOC
   speex_warning("at speex_free");
#endif
}
#define OVERRIDE_SPEEX_FREE_SCRATCH
void speex_free_scratch (void *ptr)
{
#ifdef VERBOSE_ALLOC
   speex_warning("at speex_free_scratch");
#endif
}

#endif    /* !MANUAL_ALLOC */
